#!/usr/bin/env python
"""Generates the pamqp/specification.py file used as a foundation for AMQP
communication.

"""
import copy
import datetime
import json
import keyword
from os import path
import textwrap
import urllib

import lxml.etree

__author__ = 'Gavin M. Roy'
__email__ = 'gavinmroy@gmail.com'
__since__ = '2011-03-31'

CODEGEN_DIR = '../codegen/'
CODEGEN_IGNORE_CLASSES = ['access']
CODEGEN_JSON = CODEGEN_DIR + 'amqp-rabbitmq-0.9.1.json'
CODEGEN_XML = CODEGEN_DIR + 'amqp0-9-1.xml'
CODEGEN_OUTPUT = '../pamqp/specification.py'
CODEGEN_JSON_URL = ('https://raw.githubusercontent.com/rabbitmq/'
                    'rabbitmq-codegen/master/amqp-rabbitmq-0.9.1.json')
CODEGEN_XML_URL = 'http://www.rabbitmq.com/resources/specs/amqp0-9-1.xml'

XPATH_ORDER = ['class', 'constant', 'method', 'field']
PREPEND = [CODEGEN_DIR + 'include.py']

# Output buffer list
output = []


def new_line(text='', indent_value=0, secondary_indent=0):
    """Append a new line to the output buffer"""
    global output

    if not text:
        output.append(text)
        return

    initial = ''.rjust(indent_value)
    secondary = ''.rjust(secondary_indent or indent_value)

    wrapper = textwrap.TextWrapper(
        width=79, drop_whitespace=True, initial_indent=initial,
        subsequent_indent=secondary)

    for value in wrapper.wrap(text.rstrip()):
        output.append(value)


def classify(text):
    """Replace the AMQP constant with a more pythonic classname"""
    parts = text.split('-')
    value = ''
    for part in parts:
        value += part.title()
    return value


def comment(text, indent_value=0, prefix='# '):
    """Append a comment to the output buffer"""
    for value in get_comments(text, indent_value + len(prefix), prefix):
        new_line(value)


def get_comments(text, indent_value=0, prefix='# '):
    """Return a list of lines for a given comment with the comment prefix"""
    indent_text = prefix.rjust(indent_value)
    values = []
    for value in textwrap.wrap(text, 79 - len(indent_text)):
        values.append(indent_text + value)
    return values


def dashify(text):
    """Replace a - with a _ for the passed in text"""
    return text.replace('-', '_')


def pep8_class_name(value):
    """Returns a class name in the proper case per PEP8"""
    return_value = []
    parts = value.split('-')
    for part in parts:
        return_value.append(part[0:1].upper() + part[1:])
    return ''.join(return_value)


def get_class_definition(cls_name, cls_list):
    """Iterates through class_list trying to match the name against what was
    passed in.

    """
    for cls_def in cls_list:
        if cls_def['name'] == cls_name:
            return cls_def


def get_documentation(search_path):
    """Find the documentation in the xpath

    :param search_path:
    :return:
    """
    search = []
    for k in XPATH_ORDER:
        if k in search_path:
            search.append('%s[@name="%s"]' % (k, search_path[k]))

    node = xml.xpath('%s/doc' % '/'.join(search))

    # Did we not find it? Look for a RabbitMQ extension
    if not node:
        node = rabbitmq.xpath('%s/doc' % '/'.join(search))

    # Look for RabbitMQ extensions of methods
    if not node and 'field' in search_path:
        node = rabbitmq.xpath('field[@name="%s"]/doc' % search_path['method'])

    # Look for RabbitMQ extensions of fields
    if not node and 'field' in search_path:
        node = rabbitmq.xpath('field[@name="%s"]/doc' % search_path['field'])

    # if we found it, strip all the whitespace
    if node:
        return ' '.join([l.strip() for l in node[0].text.split('\n')]).strip()


def get_label(search_path):
    """Look to see if documented & if so, provide the doc as a comment

    :param search_path:
    :return:

    """
    search = []
    for k in XPATH_ORDER:
        if k in search_path:
            search.append('%s[@name="%s"]' % (k, search_path[k]))

    node = xml.xpath('%s' % '/'.join(search))

    if not node:
        node = rabbitmq.xpath('%s' % '/'.join(search))

    # Did it have a value by default?
    if node and 'label' in node[0].attrib:
        return (node[0].attrib['label'][0:1].upper() +
                node[0].attrib['label'][1:])
    elif node and node[0].text:
        return (node[0].text.strip()[0:1].upper() +
                node[0].text.strip()[1:].strip())

    # Look in domains
    if 'field' in search_path:
        node = xml.xpath('//amqp/domain[@name="%s"]' % search_path['field'])
        if node and 'label' in node[0].attrib:
            return (node[0].attrib['label'][0:1].upper() +
                    node[0].attrib['label'][1:])

    # Look for RabbitMQ extensions of fields
    if 'field' in search_path:
        node = rabbitmq.xpath('field[@name="%s"]' % search_path['field'])
        if node and 'label' in node[0].attrib:
            return (node[0].attrib['label'][0:1].upper() +
                    node[0].attrib['label'][1:])
        elif node and node[0].text:
            return (node[0].text.strip()[0:1].upper() +
                    node[0].text.strip()[1:].strip())

    print('Label could not find %r' % search_path)


def argument_name(n):
    """Returns a valid python argument name for the AMQP argument passed in

    :param str n: The argument name

    """
    value = n.replace('-', '_')
    if value in keyword.kwlist:
        value += '_'
    return value


def get_argument_type_doc(arg):
    """Get the type of the argument for the doc

    :param arg:
    :return: str
    """
    if 'domain' in arg:
        for d, dt in amqp['domains']:
            if arg['domain'] == d:
                arg['type'] = dt
                break
    if 'type' in arg:
        if arg['type'] == 'bit':
            return 'bool'
        elif arg['type'] == 'long':
            return 'int/long'
        elif arg['type'] == 'longlong':
            return 'int/long'
        elif arg['type'] == 'longstr':
            return 'str'
        elif arg['type'] == 'octet':
            return 'int'
        elif arg['type'] == 'short':
            return 'int'
        elif arg['type'] == 'shortstr':
            return 'str'
        elif arg['type'] == 'table':
            return 'dict'
        elif arg['type'] == 'timestamp':
            return 'struct_time'
    return 'Unknown'


def get_argument_type(arg):
    """Get the argument type.

    :param arg:
    :return: str

    """
    if 'domain' in arg:
        for d, dt in amqp['domains']:
            if arg['domain'] == d:
                arg['type'] = dt
                break
    if 'type' in arg:
        return arg['type']
    return 'Unknown'


def new_function(function_name, args_in, indent_value=0):
    """Create a new function

    :param function_name:
    :param args_in:
    :param indent_value:
    :return:

    """
    global output

    args = ['self']
    for a in args_in:
        n = argument_name(a['name'])
        if 'default-value' in a and a['default-value'] != '':
            if a['default-value'] in keyword.kwlist or \
                    isinstance(a['default-value'], (bool, int)):
                value = a['default-value']
            else:
                if isinstance(a['default-value'], basestring):
                    value = "'%s'" % str(a['default-value'])
                else:
                    value = '%r' % a['default-value']
        else:
            if a['type'][-3:] == 'str':
                value = "''"
            elif a['type'] in ['short', 'long']:
                value = 0
            else:
                value = 'None'
        if value == '{}':
            if a['type'][-3:] == 'str':
                value = "''"
            elif a['type'] in ['short', 'long']:
                value = 0
            else:
                value = 'None'

        args.append('%s=%s' % (n, value))

    # Get the definition line built
    def_line = 'def %s(%s):' % (function_name, ', '.join(args))

    # Build the output of it with wrapping
    indent_str = ''.join(
        [' ' for _x in range(indent_value + len(function_name) + 5)])
    lines = textwrap.wrap(
        ''.join([' ' for _x in range(indent_value)]) + def_line, 79,
        subsequent_indent=indent_str)

    for l in lines:
        new_line(l)


# Check to see if we have the codegen json file in this directory
if not path.exists(CODEGEN_JSON):

    # Retrieve the codegen archive
    print('Downloading codegen JSON file to %s.' % CODEGEN_JSON)
    json_data = urllib.urlopen(CODEGEN_JSON_URL)

    # Write out the JSON file
    with open(CODEGEN_JSON, 'w') as handle:
        handle.write(json_data.read())

    json_data.close()

# Read in the codegen JSON file
with open(CODEGEN_JSON, 'r') as handle:
    amqp = json.load(handle)

# Check to see if we have the codegen xml file in this directory
if not path.exists(CODEGEN_XML):

    # Retrieve the codegen XML definition
    print('Downloading codegen XML file.')
    handle = urllib.urlopen(CODEGEN_XML_URL)
    xml_content = handle.read()

    # Write out the XML file
    with open(CODEGEN_XML, 'w') as handle:
        handle.write(xml_content)

# Read in the codegen XML file
with open(CODEGEN_XML, 'r') as handle:
    amqp_xml = lxml.etree.parse(handle)
    xml = amqp_xml.xpath('//amqp')[0]

# Read in the codegen RabbitMQ Extension XML file
with open(CODEGEN_DIR + 'extensions.xml', 'r') as handle:
    rabbitmq_xml = lxml.etree.parse(handle)
    rabbitmq = rabbitmq_xml.xpath('//rabbitmq')[0]

# Start the output
output = ['''"""%s

Auto-generated AMQP Support Module

WARNING: DO NOT EDIT. To Generate run tools/codegen.py

"""
__since__ = '%s'

import struct
import warnings

from pamqp import decode, encode
''' % (CODEGEN_OUTPUT.split('/')[-1], datetime.date.today().isoformat())]

new_line()

# AMQP Version Header
comment('AMQP Protocol Version')
new_line('VERSION = (%i, %i, %i)' % (
    amqp['major-version'], amqp['minor-version'], amqp['revision']))
new_line()

# Defaults
comment('RabbitMQ Defaults')
new_line("DEFAULT_HOST = 'localhost'")
new_line('DEFAULT_PORT = %i' % amqp['port'])
new_line("DEFAULT_USER = 'guest'")
new_line("DEFAULT_PASS = 'guest'")
new_line("DEFAULT_VHOST = '/'")
new_line()

# Constant
comment('AMQP Constants')
for constant in amqp['constants']:
    if 'class' not in constant:
        # Look to see if documented & if so, provide the doc as a comment
        doc = get_documentation({'constant': constant['name'].lower()})
        if doc:
            comment(doc)
        new_line('%s = %i' % (dashify(constant['name']), constant['value']))
new_line()
comment('Not included in the spec XML or JSON files.')
new_line('FRAME_MAX_SIZE = 131072')
new_line()

# Data types
data_types = []
domains = []
for domain, data_type in amqp['domains']:
    if domain == data_type:
        data_types.append("              '%s'," % domain)
    else:
        doc = get_documentation({'domain': domain})
        if doc:
            comments = get_comments(doc, 18)
            for line in comments:
                domains.append(line)
        domains.append("           '%s': '%s'," % (domain, data_type))

comment('AMQP data types')
data_types[0] = data_types[0].replace('              ',
                                      'DATA_TYPES = [')
data_types[-1] = data_types[-1].replace(',', ']')
output += data_types
new_line()

comment('AMQP domains')
domains[0] = domains[0].replace('           ',
                                'DOMAINS = {')

domains[-1] = domains[-1].replace(',', '}')
output += domains
new_line()

comment('Other constants')
# Deprecation Warning

AMQP_VERSION = ('-'.join([str(amqp['major-version']),
                          str(amqp['minor-version']),
                          str(amqp['revision'])]))
DEPRECATION_WARNING = 'This command is deprecated in AMQP %s' % AMQP_VERSION

new_line("DEPRECATION_WARNING = '%s'" % DEPRECATION_WARNING)
new_line()


# Prepend the content from the files specified
for filename in PREPEND:
    with open(filename, 'r') as handle:
        content = handle.read()
        for line in content.split('\n'):
            new_line(line)

# Warnings and Exceptions
new_line()
comment('AMQP Errors')
errors = {}
for constant in amqp['constants']:
    if 'class' in constant:
        class_name = classify(constant['name'])
        if constant['class'] == 'soft-error':
            extends = 'Warning'
        elif constant['class'] == 'hard-error':
            extends = 'Exception'
        else:
            raise ValueError('Unexpected class: %s', constant['class'])
        new_line('class AMQP%s(%s):' % (class_name, extends))
        new_line('    """')
        # Look to see if documented & if so, provide the doc as a comment
        doc = get_documentation({'constant': constant['name'].lower()})
        if doc:
            comment(doc, 4, '')
        else:
            if extends == 'Warning':
                new_line('    Undocumented AMQP Soft Error')
            else:
                new_line('    Undocumented AMQP Hard Error')
        new_line()
        new_line('    """')
        new_line("    name = '%s'" % constant['name'])
        new_line('    value = %i' % constant['value'])
        new_line()
        new_line()
        errors[constant['value']] = class_name

# Error mapping to class
error_lines = []
for error_code in errors.keys():
    error_lines.append(
        '          %i: AMQP%s,' % (error_code, errors[error_code]))
comment('AMQP Error code to class mapping')
error_lines[0] = error_lines[0].replace('          ', 'ERRORS = {')
error_lines[-1] = error_lines[-1].replace(',', '}')
output += error_lines

# Get the pamqp class list so we can sort it
class_list = []
for amqp_class in amqp['classes']:
    if amqp_class['name'] not in CODEGEN_IGNORE_CLASSES:
        class_list.append(amqp_class['name'])

# Sort them alphabetically
# class_list.sort()

new_line()
comment('AMQP Classes and Methods')
new_line()

for class_name in class_list:

    indent = 4

    # Get the class from our JSON file
    definition = get_class_definition(class_name, amqp['classes'])
    new_line()
    new_line('class %s(object):' % pep8_class_name(class_name))

    doc = get_documentation({'class': class_name})
    label = get_label({'class': class_name}) or 'Undefined label'
    if doc:
        new_line('"""' + label, indent)
        new_line()
        comment(doc, indent, '')
        new_line()
        new_line('"""', indent)

    new_line('__slots__ = []', indent)
    new_line()
    comment('AMQP Class Number and Mapping Index', indent)
    new_line('frame_id = %i' % definition['id'], indent)
    new_line('index = 0x%08X' % (definition['id'] << 16), indent)
    new_line()

    # We use this later down in methods to get method xml to look for stuff
    # that is not in the JSON spec file beyond docs
    class_xml = xml.xpath('//amqp/class[@name="%s"]' % class_name)

    # Build the list of methods
    methods = []
    for method in definition['methods']:
        new_line('class %s(Frame):' %
                 pep8_class_name(method['name']), indent)
        indent += 4

        # No Confirm in AMQP spec
        if class_xml:
            doc = get_documentation({'class': class_name,
                                     'method': method['name']})
            label = get_label({'class': class_name,
                               'method': method['name']}) or 'Undefined label'
            if doc:
                new_line('"""%s' % label, indent)
                new_line()
                comment(doc, indent, '')
                new_line()
                new_line('"""', indent)

        # Get the method's XML node
        method_xml = None
        if class_xml:
            method_xml = class_xml[0].xpath('method[@name="%s"]' %
                                            method['name'])

        comment('AMQP Method Number and Mapping Index', indent)
        new_line('frame_id = %i' % method['id'], indent)
        index_value = definition['id'] << 16 | method['id']
        new_line('index = 0x%08X' % index_value, indent)
        new_line("name = '%s.%s'" % (pep8_class_name(class_name),
                                     pep8_class_name(method['name'])),
                 indent)
        # Add an attribute that signifies if it's a sync command
        new_line()
        comment('Specifies if this is a synchronous AMQP method', indent)
        new_line('synchronous = %s' % method.get('synchronous', False),
                 indent)

        # Add an attribute that signifies if it's a sync command
        if method.get('synchronous'):
            responses = []
            if method_xml:
                for response in method_xml[0].iter('response'):

                    response_name = "'%s.%s'" %\
                                    (pep8_class_name(class_name),
                                     pep8_class_name(response.attrib['name']))
                    responses.append(response_name)
            if not responses:
                responses.append("'%s.%sOk'" %
                                 (pep8_class_name(class_name),
                                  pep8_class_name(method['name'])))
            new_line()
            comment('Valid responses to this method', indent)
            new_line('valid_responses = [%s]' % ', '.join(responses),
                     indent)
        new_line()

        arguments = []
        type_keyword = False
        for argument in method['arguments']:
            name = argument_name(argument['name'])
            if name == 'type' and class_name == 'exchange':
                name = 'exchange_type'
                type_keyword = True
            arguments.append("'%s'," % name)

        if arguments:
            comment('AMQP Method Attributes', indent)
            arguments[-1] = arguments[-1].replace(',', ']')
            new_line('__slots__ = [' + arguments.pop(0), indent)
            for line in arguments:
                new_line(line, indent + 13)
            new_line()

        if method['arguments']:
            comment('Class Attribute Types', indent)
            for argument in method['arguments']:
                name = argument_name(argument['name'])
                if name == 'type' and class_name == 'exchange':
                    name = 'exchange_type'
                new_line("_%s = '%s'" % (name, get_argument_type(argument)),
                         indent)
            new_line()

        # Function definition
        arguments = copy.deepcopy(method['arguments'])
        for offset in range(0, len(arguments)):
            if arguments[offset]['name'] == 'type' and \
                    class_name == 'exchange':
                arguments[offset]['name'] = 'exchange_type'

        if arguments:
            new_function('__init__', arguments, indent)
            indent += 4
            new_line('"""Initialize the %s.%s class' %
                     (pep8_class_name(class_name),
                      pep8_class_name(method['name'])),
                     indent)

            if type_keyword:
                new_line()
                new_line('Note that the AMQP type argument is referred to as '
                         '"%s_type" ' % class_name, indent)
                new_line('to not conflict with the Python type keyword.',
                         indent)

            # List the arguments in the docblock
            new_line()
            for argument in method['arguments']:
                name = argument_name(argument['name'])

                if name == 'type' and class_name == 'exchange':
                    name = 'exchange_type'

                label = get_label({'class': class_name,
                                   'method': method['name'],
                                   'field': argument['name']})
                if label:
                    new_line(':param %s %s: %s' %
                             (get_argument_type_doc(argument), name, label),
                             indent, indent + 4)
                else:
                    new_line(
                        ':param %s %s:' % (
                            get_argument_type_doc(argument),
                            argument['name']),
                        indent, indent + 4)

            # Note the deprecation warning in the docblock
            if method_xml and 'deprecated' in method_xml[0].attrib and \
               method_xml[0].attrib['deprecated']:
                deprecated = True
                new_line()
                new_line(
                    '.. deprecated:: %s' % AMQP_VERSION, indent)
                new_line(DEPRECATION_WARNING, indent + 4)
            else:
                deprecated = False

            new_line()
            new_line('"""', indent)

            # Create assignments from the arguments to attributes of the object
            for argument in method['arguments']:
                name = argument_name(argument['name'])

                if name == 'type' and class_name == 'exchange':
                    name = 'exchange_type'

                doc = get_label({'class': class_name,
                                 'method': method['name'],
                                 'field': argument['name']})
                if doc:
                    comment(doc, indent)

                if (isinstance(argument.get('default-value'), dict) and
                        not argument.get('default-value')):
                    new_line('self.%s = %s or {}' % (name, name), indent)
                else:
                    new_line('self.%s = %s' % (name, name), indent)
                new_line()

            # Check if we're deprecated and warn if so
            if deprecated:
                comment(DEPRECATION_WARNING, indent)
                new_line('warnings.warn(DEPRECATION_WARNING, '
                         'category=DeprecationWarning)', indent)
                new_line()

            # End of function
            indent -= 4

        # End of class
        indent -= 4

    if 'properties' in definition and definition['properties']:
        new_line('class Properties(PropertiesBase):', indent)
        indent += 4
        comment('"""Content Properties"""', indent, '')
        new_line()

        new_line("name = '%s.Properties'" % pep8_class_name(class_name),
                 indent)
        new_line()

        new_line("__slots__ = ['%s'," %
                 argument_name(definition['properties'][0]['name']),
                 indent)
        for argument in definition['properties'][1:-1]:
            name = argument_name(argument['name'])
            if name == 'type':
                name = 'message_type'
            new_line("'%s'," % name, indent + 13)
        new_line("'%s']" % argument_name(definition['properties'][-1]['name']),
                 indent + 13)
        new_line()

        comment('Flag Values', indent)
        flag_value = 15
        new_line("flags = {'%s': %i," %
                 (argument_name(definition['properties'][0]['name']),
                  1 << flag_value), indent)
        for argument in definition['properties'][1:-1]:
            name = argument_name(argument['name'])
            if name == 'type':
                name = 'message_type'
            flag_value -= 1
            new_line("'%s': %i," % (name, 1 << flag_value), indent + 9),
        flag_value -= 1
        new_line("'%s': %i}" %
                 (argument_name(definition['properties'][-1]['name']),
                  1 << flag_value), indent + 9)
        new_line()

        comment('Class Attribute Types', indent)
        for argument in definition['properties']:
            name = argument_name(argument['name'])
            if name == 'type':
                name = 'message_type'
            new_line("_%s = '%s'" % (name, get_argument_type(argument)),
                     indent)
        new_line()
        new_line('frame_id = %i' % definition['id'], indent)
        new_line('index = 0x%04X' % definition['id'], indent)
        new_line()

        # Function definition
        properties = copy.deepcopy(definition['properties'])
        for offset in range(0, len(properties)):
            if properties[offset]['name'] == 'type':
                properties[offset]['name'] = 'message_type'

        new_function('__init__', properties, indent)
        indent += 4
        new_line('"""Initialize the %s.Properties class' %
                 pep8_class_name(class_name),
                 indent)
        new_line()
        new_line('Note that the AMQP property type is named message_type as '
                 'to ', indent)
        new_line('not conflict with the Python type keyword', indent)
        # List the arguments in the docblock
        new_line()
        for argument in definition['properties']:
            name = argument_name(argument['name'])
            if name == 'type':
                name = 'message_type'
            label = get_label({'class': class_name,
                               'field': argument['name']})
            if label:
                line = ':param %s %s: %s' % (get_argument_type_doc(argument),
                                             name, label or None)
                new_line(line.strip(), indent)

        new_line()
        new_line('"""', indent)

        # Create assignments from the arguments to attributes of the object
        for argument in definition['properties']:
            name = argument_name(argument['name'])
            if name == 'type':
                name = 'message_type'
            doc = get_label({'class': class_name,
                             'field': argument['name']})
            if doc:
                comment(doc, indent)

            new_line('self.%s = %s' % (name, name), indent)
            new_line()

        # End of function
        indent -= 4

new_line()
comment('AMQP Class.Method Index Mapping')
mapping = []
for amqp_class in amqp['classes']:
    if amqp_class['name'] not in CODEGEN_IGNORE_CLASSES:
        for method in amqp_class['methods']:
            key = amqp_class['id'] << 16 | method['id']
            mapping.append(('                 0x%08X: %s.%s,' %
                            (key,
                             pep8_class_name(amqp_class['name']),
                             pep8_class_name(method['name']))))
mapping[0] = mapping[0].replace('                 ',
                                'INDEX_MAPPING = {')
mapping[-1] = mapping[-1].replace(',', '}')
output += mapping
new_line()

# Spit out the file
output_string = '\n'.join(output)
with open(CODEGEN_OUTPUT, 'w') as handle:
    handle.write(output_string)



# -*- encoding: utf-8 -*-
import unittest

from pamqp import decode, encode


class EncodeDecodeTests(unittest.TestCase):

    def test_encode_decode_field_table_long_keys(self):
        """Encoding and decoding a field_table with too long keys."""
        # second key is 126 A's + \N{PILE OF POO}
        data = {'A' * 256: 1,
                ((b'A' * 128) + b'\xf0\x9f\x92\xa9').decode('utf-8'): 2}
        encoded = encode.field_table(data)
        decoded = decode.field_table(encoded)[1]
        self.assertIn('A' * 128, decoded)



"""
Deprecating the codec subpackage

"""
import warnings

from pamqp import decode, encode

warnings.warn('Imports for decode/encode are no longer under codec',
              DeprecationWarning)
__all__ = ['decode', 'encode']



# -*- encoding: utf-8 -*-
import time
import unittest

from pamqp import body, frame, header, PYTHON3, specification

if PYTHON3:
    long = int
    basestring = str
    unicode = None


class DemarshalingTests(unittest.TestCase):
    def test_protocol_header(self):
        # Decode the frame and validate lengths
        frame_data = b'AMQP\x00\x00\t\x01'
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 8,
            'Bytes consumed did not match expectation: %i, %i' % (consumed, 8))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'ProtocolHeader',
                         ('Frame was of wrong type, expected ProtocolHeader, '
                          'received %s' % frame_obj.name))

        self.assertEqual((frame_obj.major_version, frame_obj.minor_version,
                          frame_obj.revision), (0, 9, 1),
                         'Protocol version is incorrect')

    def test_heartbeat(self):
        frame_data = b'\x08\x00\x00\x00\x00\x00\x00\xce'
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 8,
            'Bytes consumed did not match expectation: %i, %i' % (consumed, 8))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Heartbeat',
                         ('Frame was of wrong type, expected Heartbeat, '
                          'received %s' % frame_obj.name))

    def test_basic_ack(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\r\x00<\x00P\x00\x00\x00\x00'
                      b'\x00\x00\x00\x01\x00\xce')
        expectation = {'multiple': False, 'delivery_tag': 1}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 21, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 21))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Ack',
                         ('Frame was of wrong type, expected Basic.Ack, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_cancel(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\r\x00<\x00\x1e\x07ctag1.0\x00'
                      b'\xce')
        expectation = {'consumer_tag': 'ctag1.0', 'nowait': False}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 21, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 21))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Cancel',
                         ('Frame was of wrong type, expected Basic.Cancel, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_cancelok(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x0c\x00<\x00\x1f\x07ctag1.0'
                      b'\xce')
        expectation = {'consumer_tag': 'ctag1.0'}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 20, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 20))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.CancelOk',
                         ('Frame was of wrong type, expected Basic.CancelOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_consume(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x18\x00<\x00\x14\x00\x00\x04'
                      b'test\x07ctag1.0\x00\x00\x00\x00\x00\xce')
        expectation = {
            'exclusive': False,
            'nowait': False,
            'no_local': False,
            'consumer_tag': 'ctag1.0',
            'queue': 'test',
            'arguments': {},
            'ticket': 0,
            'no_ack': False
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 32, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 32))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Consume',
                         ('Frame was of wrong type, expected Basic.Consume, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_consumeok(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x0c\x00<\x00\x15\x07ctag1.0'
                      b'\xce')
        expectation = {'consumer_tag': 'ctag1.0'}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 20, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 20))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.ConsumeOk',
                         ('Frame was of wrong type, expected Basic.ConsumeOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_deliver(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x1b\x00<\x00<\x07ctag1.0'
                      b'\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x04test\xce')

        expectation = {
            'consumer_tag': 'ctag1.0',
            'delivery_tag': 1,
            'redelivered': False,
            'exchange': '',
            'routing_key': 'test'
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 35, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 35))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Deliver',
                         ('Frame was of wrong type, expected Basic.Deliver, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_get(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x0c\x00<\x00F\x00\x00\x04'
                      b'test\x00\xce')
        expectation = {'queue': 'test', 'ticket': 0, 'no_ack': False}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 20, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 20))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Get',
                         ('Frame was of wrong type, expected Basic.Get, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_getempty(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x05\x00<\x00H\x00\xce'
        expectation = {'cluster_id': ''}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 13, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 13))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.GetEmpty',
                         ('Frame was of wrong type, expected Basic.GetEmpty, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_getok(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x17\x00<\x00G\x00\x00\x00\x00'
                      b'\x00\x00\x00\x10\x00\x00\x04test\x00\x00\x12\x06\xce')
        expectation = {
            'message_count': 4614,
            'redelivered': False,
            'routing_key': 'test',
            'delivery_tag': 16,
            'exchange': ''
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 31, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 31))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.GetOk',
                         ('Frame was of wrong type, expected Basic.GetOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_nack(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\r\x00<\x00x\x00\x00\x00\x00'
                      b'\x00\x00\x00\x01\x00\xce')
        expectation = {'requeue': False, 'multiple': False, 'delivery_tag': 1}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 21, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 21))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Nack',
                         ('Frame was of wrong type, expected Basic.Nack, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_content_header(self):
        result = frame.unmarshal(frame.marshal(header.ContentHeader(0, 100),
                                               1))
        self.assertEqual(result[2].body_size, 100)

    def test_content_body(self):
        result = frame.unmarshal(frame.marshal(body.ContentBody(b'TEST'), 1))
        self.assertEqual(len(result[2]), 4)

    def test_basic_properties(self):
        encoded_properties = (b'\xff\xfc\x10application/json\x04gzip\x00\x00'
                              b'\x00\x1d\x03bazS\x00\x00\x00\x08Test \xe2\x9c'
                              b'\x88\x03fooS\x00\x00\x00\x03bar\x01\x00$a5304'
                              b'5ef-f174-4621-9ff2-ac0b8fbe6e4a\x12unmarshali'
                              b'ng_tests\n1345274026$746a1902-39dc-47cf-9471-'
                              b'9feecda35660\x00\x00\x00\x00Pj\xb9\x07\x08uni'
                              b'ttest\x04pika\x18frame_unmarshaling_tests\x00')
        properties = {
            'content_type': 'application/json',
            'content_encoding': 'gzip',
            'headers': {
                'foo': b'bar',
                'baz': b'Test \xe2\x9c\x88'
            },
            'delivery_mode': 1,
            'priority': 0,
            'correlation_id': 'a53045ef-f174-4621-9ff2-ac0b8fbe6e4a',
            'reply_to': 'unmarshaling_tests',
            'expiration': '1345274026',
            'message_id': '746a1902-39dc-47cf-9471-9feecda35660',
            'timestamp': time.struct_time((2012, 10, 2, 9, 51, 3, 1, 276, 0)),
            'message_type': 'unittest',
            'user_id': 'pika',
            'app_id': 'frame_unmarshaling_tests',
            'cluster_id': ''
        }

        obj = header.ContentHeader()
        offset, flags = obj._get_flags(encoded_properties)

        # Decode the frame and validate lengths
        obj = specification.Basic.Properties()
        obj.unmarshal(flags, encoded_properties[offset:])

        for key in properties:
            self.assertEqual(
                getattr(obj, key), properties[key],
                '%r value of %r did not match expectation of %r' %
                (key, getattr(obj, key), properties[key]))

    def test_basic_publish(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\r\x00<\x00(\x00\x00\x00'
                      b'\x04test\x00\xce')
        expectation = {
            'ticket': 0,
            'mandatory': False,
            'routing_key': 'test',
            'immediate': False,
            'exchange': ''
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 21, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 21))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Publish',
                         ('Frame was of wrong type, expected Basic.Publish, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_qos(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x0b\x00<\x00\n\x00\x00\x00'
                      b'\x00\x00\x01\x00\xce')
        expectation = {
            'prefetch_count': 1,
            'prefetch_size': 0,
            'global_': False
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 19, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 19))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Qos',
                         ('Frame was of wrong type, expected Basic.Qos, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_qosok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00<\x00\x0b\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.QosOk',
                         ('Frame was of wrong type, expected Basic.QosOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_recover(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x05\x00<\x00n\x00\xce'
        expectation = {'requeue': False}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 13, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 13))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Recover',
                         ('Frame was of wrong type, expected Basic.Recover, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_recoverasync(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x05\x00<\x00d\x00\xce'
        expectation = {'requeue': False}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)
        self.assertEqual(
            consumed, 13, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 13))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(
            frame_obj.name, 'Basic.RecoverAsync',
            ('Frame was of wrong type, expected Basic.RecoverAsync, '
             'received %s' % frame_obj.name))

        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_recoverok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00<\x00o\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.RecoverOk',
                         ('Frame was of wrong type, expected Basic.RecoverOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_reject(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\r\x00<\x00Z\x00\x00\x00\x00'
                      b'\x00\x00\x00\x10\x01\xce')
        expectation = {'requeue': True, 'delivery_tag': 16}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 21, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 21))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Reject',
                         ('Frame was of wrong type, expected Basic.Reject, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_basic_return(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00"\x00<\x002\x00\xc8\x0f'
                      b'Normal shutdown\x03foo\x07foo.bar\xce')
        expectation = {
            'reply_code': 200,
            'reply_text': 'Normal shutdown',
            'routing_key': 'foo.bar',
            'exchange': 'foo'
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 42, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 42))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Basic.Return',
                         ('Frame was of wrong type, expected Basic.Return, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_channel_close(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x1a\x00\x14\x00(\x00\xc8\x0f'
                      b'Normal shutdown\x00\x00\x00\x00\xce')
        expectation = {
            'class_id': 0,
            'method_id': 0,
            'reply_code': 200,
            'reply_text': 'Normal shutdown'
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 34, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 34))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Channel.Close',
                         ('Frame was of wrong type, expected Channel.Close, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_channel_closeok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00\x14\x00)\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Channel.CloseOk',
                         ('Frame was of wrong type, expected Channel.CloseOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_channel_flow(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x05\x00\x14\x00\x14\x01\xce'
        expectation = {'active': True}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 13, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 13))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Channel.Flow',
                         ('Frame was of wrong type, expected Channel.Flow, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_channel_flowok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x05\x00\x14\x00\x15\x01\xce'
        expectation = {'active': True}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 13, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 13))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Channel.FlowOk',
                         ('Frame was of wrong type, expected Channel.FlowOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_channel_open(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x05\x00\x14\x00\n\x00\xce'
        expectation = {'out_of_band': ''}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 13, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 13))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Channel.Open',
                         ('Frame was of wrong type, expected Channel.Open, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_channel_openok(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x08\x00\x14\x00\x0b\x00\x00'
                      b'\x00\x00\xce')
        expectation = {'channel_id': b''}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 16, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 16))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Channel.OpenOk',
                         ('Frame was of wrong type, expected Channel.OpenOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_confirm_select(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x05\x00U\x00\n\x00\xce'
        expectation = {'nowait': False}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 13, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 13))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Confirm.Select',
                         ('Frame was of wrong type, expected Confirm.Select, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_confirm_selectok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00U\x00\x0b\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Confirm.SelectOk',
                         ('Frame was of wrong type, expected '
                          'Confirm.SelectOk, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_connection_close(self):
        frame_data = (b'\x01\x00\x00\x00\x00\x00\x1a\x00\n\x002\x00\xc8\x0f'
                      b'Normal shutdown\x00\x00\x00\x00\xce')
        expectation = {
            'class_id': 0,
            'method_id': 0,
            'reply_code': 200,
            'reply_text': 'Normal shutdown'
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 34, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 34))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Connection.Close',
                         ('Frame was of wrong type, expected '
                          'Connection.Close, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_connection_closeok(self):
        frame_data = b'\x01\x00\x00\x00\x00\x00\x04\x00\n\x003\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Connection.CloseOk',
                         ('Frame was of wrong type, expected '
                          'Connection.CloseOk, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_connection_open(self):
        frame_data = (b'\x01\x00\x00\x00\x00\x00\x08\x00\n\x00(\x01/\x00'
                      b'\x01\xce')
        expectation = {'insist': True, 'capabilities': '', 'virtual_host': '/'}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 16, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 16))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Connection.Open',
                         ('Frame was of wrong type, expected Connection.Open, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_connection_openok(self):
        frame_data = b'\x01\x00\x00\x00\x00\x00\x05\x00\n\x00)\x00\xce'
        expectation = {'known_hosts': ''}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 13, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 13))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Connection.OpenOk',
                         ('Frame was of wrong type, expected '
                          'Connection.OpenOk, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_connection_secure(self):
        frame_data = (b'\x01\x00\x00\x00\x00\x00\x08\x00\n\x00\x14\x00\x00'
                      b'\x00\x00\xce')
        expectation = {'challenge': b''}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 16, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 16))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Connection.Secure',
                         ('Frame was of wrong type, expected '
                          'Connection.Secure, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_connection_secureok(self):
        frame_data = (b'\x01\x00\x00\x00\x00\x00\x08\x00\n\x00\x15\x00\x00'
                      b'\x00\x00\xce')
        expectation = {'response': b''}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 16, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 16))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Connection.SecureOk',
                         ('Frame was of wrong type, expected '
                          'Connection.SecureOk, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_connection_start(self):
        frame_data = (b'\x01\x00\x00\x00\x00\x01G\x00\n\x00\n\x00\t'
                      b'\x00\x00\x01"\x0ccapabilitiesF\x00\x00\x00X'
                      b'\x12publisher_confirmst\x01\x1aexchange_exc'
                      b'hange_bindingst\x01\nbasic.nackt\x01\x16con'
                      b'sumer_cancel_notifyt\x01\tcopyrightS\x00'
                      b'\x00\x00$Copyright (C) 2007-2011 VMware, I'
                      b'nc.\x0binformationS\x00\x00\x005Licensed u'
                      b'nder the MPL.  See http://www.rabbitmq.com'
                      b'/\x08platformS\x00\x00\x00\nErlang/OTP\x07'
                      b'productS\x00\x00\x00\x08RabbitMQ\x07versio'
                      b'nS\x00\x00\x00\x052.6.1\x00\x00\x00\x0ePLA'
                      b'IN AMQPLAIN\x00\x00\x00\x05en_US\xce')
        expectation = {
            'server_properties': {
                'information': (b'Licensed under the MPL.  '
                                b'See http://www.rabbitmq.com/'),
                'product':
                b'RabbitMQ',
                'copyright':
                b'Copyright (C) 2007-2011 VMware, Inc.',
                'capabilities': {
                    'exchange_exchange_bindings': True,
                    'consumer_cancel_notify': True,
                    'publisher_confirms': True,
                    'basic.nack': True
                },
                'platform':
                b'Erlang/OTP',
                'version':
                b'2.6.1'
            },
            'version_minor': 9,
            'mechanisms': b'PLAIN AMQPLAIN',
            'locales': b'en_US',
            'version_major': 0
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 335, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 335))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Connection.Start',
                         ('Frame was of wrong type, expected '
                          'Connection.Start, received %s' % frame_obj.name))
        self.maxDiff = 32768

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_connection_startok(self):
        frame_data = (b'\x01\x00\x00\x00\x00\x00\xf4\x00\n\x00\x0b'
                      b'\x00\x00\x00\xd0\x08platformS\x00\x00\x00'
                      b'\x0cPython 2.7.1\x07productS\x00\x00\x00'
                      b'\x1aPika Python Client Library\x07versionS'
                      b'\x00\x00\x00\n0.9.6-pre0\x0ccapabilitiesF'
                      b'\x00\x00\x00;\x16consumer_cancel_notifyt'
                      b'\x01\x12publisher_confirmst\x01\nbasic.nack'
                      b't\x01\x0binformationS\x00\x00\x00\x1aSee '
                      b'http://pika.github.com\x05PLAIN\x00\x00'
                      b'\x00\x0c\x00guest\x00guest\x05en_US\xce')
        expectation = {
            'locale': 'en_US',
            'mechanism': 'PLAIN',
            'client_properties': {
                'platform': b'Python 2.7.1',
                'product': b'Pika Python Client Library',
                'version': b'0.9.6-pre0',
                'capabilities': {
                    'consumer_cancel_notify': True,
                    'publisher_confirms': True,
                    'basic.nack': True
                },
                'information': b'See http://pika.github.com'
            },
            'response': b'\x00guest\x00guest'
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 252, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 252))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Connection.StartOk',
                         ('Frame was of wrong type, expected '
                          'Connection.StartOk, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_connection_tune(self):
        frame_data = (b'\x01\x00\x00\x00\x00\x00\x0c\x00\n\x00\x1e\x00\x00'
                      b'\x00\x02\x00\x00\x00\x00\xce')
        expectation = {'frame_max': 131072, 'channel_max': 0, 'heartbeat': 0}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 20, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 20))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Connection.Tune',
                         ('Frame was of wrong type, expected Connection.Tune, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_connection_tuneok(self):
        frame_data = (b'\x01\x00\x00\x00\x00\x00\x0c\x00\n\x00\x1f\x00\x00'
                      b'\x00\x02\x00\x00\x00\x00\xce')
        expectation = {'frame_max': 131072, 'channel_max': 0, 'heartbeat': 0}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 20, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 20))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Connection.TuneOk',
                         ('Frame was of wrong type, expected '
                          'Connection.TuneOk, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_content_body_frame(self):
        frame_data = (b'\x03\x00\x00\x00\x00\x00"Hello World #0:1316899165.'
                      b'75516605\xce')

        expectation = b'Hello World #0:1316899165.75516605'

        # Decode the frame and validate lengths
        consumed, channel, data = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 42, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 42))

        self.assertEqual(
            channel, 0,
            'Channel number did not match expectation: %i, %i' % (channel, 0))

        # Validate the unmarshaled data matches the expectation
        self.assertEqual(data.value, expectation)

    def test_exchange_bind(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x15\x00(\x00\x1e\x00\x00'
                      b'\x00\x00\x07foo.bar\x00\x00\x00\x00\x00\xce')
        expectation = {
            'arguments': {},
            'source': '',
            'ticket': 0,
            'destination': '',
            'nowait': False,
            'routing_key': 'foo.bar'
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 29, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 29))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Exchange.Bind',
                         ('Frame was of wrong type, expected Exchange.Bind, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_exchange_bindok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00(\x00\x1f\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Exchange.BindOk',
                         ('Frame was of wrong type, expected Exchange.BindOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_exchange_declare(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00%\x00(\x00\n\x00\x00\x12pika_'
                      b'test_exchange\x06direct\x00\x00\x00\x00\x00\xce')
        expectation = {
            'nowait': False,
            'exchange': 'pika_test_exchange',
            'durable': False,
            'passive': False,
            'internal': False,
            'arguments': {},
            'ticket': 0,
            'exchange_type': 'direct',
            'auto_delete': False
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 45, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 45))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Exchange.Declare',
                         ('Frame was of wrong type, expected '
                          'Exchange.Declare, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_exchange_declareok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00(\x00\x0b\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Exchange.DeclareOk',
                         ('Frame was of wrong type, expected Exchange.'
                          'DeclareOk, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_exchange_delete(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x1a\x00(\x00\x14\x00\x00\x12'
                      b'pika_test_exchange\x00\xce')
        expectation = {
            'ticket': 0,
            'if_unused': False,
            'nowait': False,
            'exchange': 'pika_test_exchange'
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 34, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 34))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Exchange.Delete',
                         ('Frame was of wrong type, expected Exchange.Delete, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_exchange_deleteok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00(\x00\x15\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Exchange.DeleteOk',
                         ('Frame was of wrong type, expected '
                          'Exchange.DeleteOk, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_exchange_unbind(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x15\x00(\x00(\x00\x00\x00'
                      b'\x00\x07foo.bar\x00\x00\x00\x00\x00\xce')
        expectation = {
            'arguments': {},
            'source': '',
            'ticket': 0,
            'destination': '',
            'nowait': False,
            'routing_key': 'foo.bar'
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 29, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 29))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Exchange.Unbind',
                         ('Frame was of wrong type, expected Exchange.Unbind, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_exchange_unbindok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00(\x003\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Exchange.UnbindOk',
                         ('Frame was of wrong type, expected '
                          'Exchange.UnbindOk, received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_queue_bind(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00?\x002\x00\x14\x00'
                      b'\x00\x0fpika_test_queue\x12pika_test_excha'
                      b'nge\x10test_routing_key\x00\x00\x00\x00\x00'
                      b'\xce')
        expectation = {
            'nowait': False,
            'exchange': 'pika_test_exchange',
            'routing_key': 'test_routing_key',
            'queue': 'pika_test_queue',
            'arguments': {},
            'ticket': 0
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 71, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 71))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Queue.Bind',
                         ('Frame was of wrong type, expected Queue.Bind, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_queue_bindok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x002\x00\x15\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Queue.BindOk',
                         ('Frame was of wrong type, expected Queue.BindOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_queue_declare(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x10\x002\x00\n\x00\x00\x04'
                      b'test\x02\x00\x00\x00\x00\xce')
        expectation = {
            'passive': False,
            'nowait': False,
            'exclusive': False,
            'durable': True,
            'queue': 'test',
            'arguments': {},
            'ticket': 0,
            'auto_delete': False
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 24, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 24))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Queue.Declare',
                         ('Frame was of wrong type, expected Queue.Declare, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_queue_declareok(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x11\x002\x00\x0b\x04test'
                      b'\x00\x00\x12\x07\x00\x00\x00\x00\xce')
        expectation = {
            'queue': 'test',
            'message_count': 4615,
            'consumer_count': 0
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 25, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 25))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Queue.DeclareOk',
                         ('Frame was of wrong type, expected Queue.DeclareOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_queue_delete(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x17\x002\x00(\x00\x00\x0f'
                      b'pika_test_queue\x00\xce')
        expectation = {
            'queue': 'pika_test_queue',
            'ticket': 0,
            'if_empty': False,
            'nowait': False,
            'if_unused': False
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 31, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 31))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Queue.Delete',
                         ('Frame was of wrong type, expected Queue.Delete, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_queue_deleteok(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x08\x002\x00)\x00\x00\x00'
                      b'\x00\xce')
        expectation = {'message_count': 0}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 16, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 16))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Queue.DeleteOk',
                         ('Frame was of wrong type, expected Queue.DeleteOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_queue_purge(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x0c\x002\x00\x1e\x00\x00'
                      b'\x04test\x00\xce')
        expectation = {'queue': 'test', 'ticket': 0, 'nowait': False}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 20, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 20))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Queue.Purge',
                         ('Frame was of wrong type, expected Queue.Purge, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_queue_purgeok(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00\x08\x002\x00\x1f\x00\x00\x00'
                      b'\x01\xce')
        expectation = {'message_count': 1}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 16, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 16))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Queue.PurgeOk',
                         ('Frame was of wrong type, expected Queue.PurgeOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_queue_unbind(self):
        frame_data = (b'\x01\x00\x01\x00\x00\x00>\x002\x002\x00\x00\x0f'
                      b'pika_test_queue\x12pika_test_exchange\x10test_routing'
                      b'_key\x00\x00\x00\x00\xce')
        expectation = {
            'queue': 'pika_test_queue',
            'arguments': {},
            'ticket': 0,
            'routing_key': 'test_routing_key',
            'exchange': 'pika_test_exchange'
        }

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 70, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 70))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Queue.Unbind',
                         ('Frame was of wrong type, expected Queue.Unbind, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_queue_unbindok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x002\x003\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Queue.UnbindOk',
                         ('Frame was of wrong type, expected Queue.UnbindOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_tx_commit(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00Z\x00\x14\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Tx.Commit',
                         ('Frame was of wrong type, expected Tx.Commit, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_tx_commitok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00Z\x00\x15\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Tx.CommitOk',
                         ('Frame was of wrong type, expected Tx.CommitOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_tx_rollback(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00Z\x00\x1e\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Tx.Rollback',
                         ('Frame was of wrong type, expected Tx.Rollback, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_tx_rollbackok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00Z\x00\x1f\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Tx.RollbackOk',
                         ('Frame was of wrong type, expected Tx.RollbackOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_tx_select(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00Z\x00\n\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Tx.Select',
                         ('Frame was of wrong type, expected Tx.Select, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)

    def test_tx_selectok(self):
        frame_data = b'\x01\x00\x01\x00\x00\x00\x04\x00Z\x00\x0b\xce'
        expectation = {}

        # Decode the frame and validate lengths
        consumed, channel, frame_obj = frame.unmarshal(frame_data)

        self.assertEqual(
            consumed, 12, 'Bytes consumed did not match expectation: %i, %i' %
            (consumed, 12))

        self.assertEqual(
            channel, 1,
            'Channel number did not match expectation: %i, %i' % (channel, 1))

        # Validate the frame name
        self.assertEqual(frame_obj.name, 'Tx.SelectOk',
                         ('Frame was of wrong type, expected Tx.SelectOk, '
                          'received %s' % frame_obj.name))

        # Validate the unmarshaled data matches the expectation
        self.assertDictEqual(dict(frame_obj), expectation)



# -*- encoding: utf-8 -*-
import unittest
import uuid

from pamqp import body, frame, header, heartbeat, specification


class MarshalingTests(unittest.TestCase):
    def test_protocol_header(self):
        expectation = b'AMQP\x00\x00\t\x01'
        response = frame.marshal(header.ProtocolHeader(), 0)
        self.assertEqual(response, expectation,
                         'ProtocolHeader did not match expectation')

    def test_basic_ack(self):
        expectation = (b'\x01\x00\x01\x00\x00\x00\r\x00<\x00P\x00\x00\x00'
                       b'\x00\x00\x00\x00\x01\x00\xce')
        frame_obj = specification.Basic.Ack(1, False)
        response = frame.marshal(frame_obj, 1)
        self.assertEqual(response, expectation,
                         'Basic.Ack did not match expectation')

    def test_basic_cancel(self):
        expectation = (b'\x01\x00\x01\x00\x00\x00\r\x00<\x00\x1e\x07'
                       b'ctag1.0\x00\xce')
        frame_obj = specification.Basic.Cancel(consumer_tag='ctag1.0')
        response = frame.marshal(frame_obj, 1)
        self.assertEqual(response, expectation,
                         'Basic.Cancel did not match expectation')

    def test_basic_cancelok(self):
        expectation = (b'\x01\x00\x01\x00\x00\x00\x0c\x00<\x00\x1f\x07'
                       b'ctag1.0\xce')
        frame_obj = specification.Basic.CancelOk(consumer_tag='ctag1.0')
        response = frame.marshal(frame_obj, 1)
        self.assertEqual(response, expectation,
                         'Basic.Cancel did not match expectation')

    def test_basic_consume(self):
        expectation = (b'\x01\x00\x01\x00\x00\x00\x15\x00<\x00\x14\x00'
                       b'\x00\x03bar\x05ctag0\x00\x00\x00\x00\x00\xce')
        frame_obj = specification.Basic.Consume(0, 'bar', 'ctag0', False,
                                                False, False, False)
        response = frame.marshal(frame_obj, 1)
        self.assertEqual(response, expectation,
                         'Basic.Consume did not match expectation')

    def test_heartbeat(self):
        expectation = b'\x08\x00\x00\x00\x00\x00\x00\xce'
        response = frame.marshal(heartbeat.Heartbeat(), 0)
        self.assertEqual(response, expectation,
                         'Heartbeat did not match expectation')

    def test_content_body(self):
        value = str(uuid.uuid4()).encode('utf-8')
        expectation = b'\x03\x00\x01\x00\x00\x00$' + value + b'\xce'
        self.assertEqual(frame.marshal(body.ContentBody(value), 1),
                         expectation)

    def test_content_header(self):
        expectation = (b'\x02\x00\x01\x00\x00\x00\x0e\x00<\x00\x00\x00'
                       b'\x00\x00\x00\x00\x00\x00\n\x00\x00\xce')
        self.assertEqual(frame.marshal(header.ContentHeader(body_size=10), 1),
                         expectation)

    def test_unknown_frame_type(self):
        with self.assertRaises(ValueError):
            frame.marshal(self, 1)



# -*- encoding: utf-8 -*-
"""AMQP Specifications and Classes"""
from pamqp.constants import PYTHON3
from pamqp.header import ContentHeader, ProtocolHeader

__author__ = 'Gavin M. Roy'
__email__ = 'gavinmroy@gmail.com'
__since__ = '2011-09-23'
__version__ = '2.3.0'
__all__ = [
    'body', 'constants', 'decode', 'encode', 'exceptions', 'frame', 'header',
    'heartbeat', 'specification', 'PYTHON3', 'ContentHeader', 'ProtocolHeader'
]
